"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var nats_1 = require("nats");
var lodash_1 = require("lodash");
var PubSub = (function () {
    function PubSub(options) {
        this.connection = nats_1.connect(options.config);
        this.logger = options.logger;
        this.subscriptions = new Map();
        this.listeners = new Map();
        this.refs = new Map();
        this.nextId = 0;
    }
    PubSub.prototype.publish = function (topic, payload) {
        this.logger.trace("publishing for queue '%s' (%j)", topic, payload);
        try {
            payload = JSON.stringify(payload);
        }
        catch (e) {
            this.logger.trace('Can not publish message: %j', payload);
            payload = '{}';
        }
        this.connection.publish(topic, payload);
        return true;
    };
    PubSub.prototype.subscribe = function (topic, onMessage) {
        var _this = this;
        var id = this.nextId++;
        var subscription = this.subscriptions.get(topic);
        var sid;
        if (!subscription) {
            sid = this.connection.subscribe(topic, function (msg) { return _this.onMessage(topic, msg); });
            this.subscriptions.set(topic, { sid: sid });
        }
        else {
            sid = subscription.sid;
        }
        this.listeners.set(id, { id: id, topic: topic, handler: onMessage, sid: sid });
        this.refs.set(topic, (this.refs.get(topic) || []).concat([id]));
        return Promise.resolve(id);
    };
    PubSub.prototype.unsubscribe = function (sid) {
        var subscription = this.listeners.get(sid);
        if (subscription) {
            var topic = subscription.topic;
            var ids = this.refs.get(topic) || [];
            lodash_1.default.remove(ids, function (id) { return id === sid; });
            if (!ids.length) {
                this.connection.unsubscribe(subscription.sid);
                this.subscriptions.delete(topic);
                this.refs.delete(topic);
                this.listeners.delete(sid);
            }
            else {
                this.refs.set(topic, ids);
            }
        }
    };
    PubSub.prototype.onMessage = function (topic, msg) {
        var _this = this;
        try {
            msg = JSON.parse(msg);
        }
        catch (e) {
            this.logger.trace('Can not parse message: %j', msg);
            msg = {};
        }
        var listeners = this.refs.get(topic) || [];
        listeners.forEach(function (sid) {
            if (_this.listeners.has(sid)) {
                _this.logger.trace("sending message to subscriber callback function '(%j)'", msg);
                var handler = _this.listeners.get(sid).handler;
                handler(msg);
            }
        });
    };
    return PubSub;
}());
exports.PubSub = PubSub;
//# sourceMappingURL=pubsub.js.map